==> plantron-1.4.4.tar.gz: plantron/arena/pt-agentes-simples.lisp <==
;; Programa: Package con agentes jugadores simples de PlanTron
;;
;; Asignatura: Introducción a la Inteligencia Artificial
;;             ETSII - ULL
;; Práctica: (lisp)
;; Profesor: Patricio García (patricio AT etsii.ull.es)
;; Fecha:    28/04/2008
;; Comentarios:
;;
;;           Incluye agentes para jugar al PlanTron simples, esto
;;           es, agentes que no tiene memoria y su comportamiento
;;           viene guiado en función del entorno que perciben en
;;           cada instante (excepto el agente-hormiga-borracha,
;;           que hace uso del generador de números aleatorios).
;;
;;           Los agentes definidos utilizan el entorno de juego
;;           PlanTron, definido principalmente por los packages
;;           PT-BASE y ENVIO.
;;
;;           Los agentes incluidos son, ordenados según la
;;           complejidad de su código, los siguientes:
;;
;;             AGENTE-MITAD
;;             AGENTE-MIMITAD
;;             AGENTE-ESPIRAL
;;             AGENTE-MURALLA
;;             AGENTE-IMITADOR
;;             AGENTE-COBARDE
;;             AGENTE-HORMIGA-BORRACHA
;;
;;           Ver la documentación existente para cada uno de ellos.
;;
;;           Para su utilización es aconsejable usar el cargador
;;           del entorno de juego PlanTron: "pt-loader.lisp".
;;
;;
;; Dependencias:
;;
;;           Hace uso del paquete que contiene la interface (ENVIO)
;;           con el control del juego (PT-BASE).
;;
;;
;; Modificaciones:
;;
;;           - Ver 0.1 (28/04/2008): Primera versión
;;
;;           - Ver 0.2 (14/04/2009): Versión curso 2008/09
;;
;;           - Ver 0.2.1 (15/04/2009): Añadidos más comentarios
;;
;;           - Ver 0.3 (24/03/2010): Versión curso 2009/10
;;
;;           - Ver 0.4 (24/03/2011): Versión curso 2010/11
;;
;; Por hacer:
;;
;;
;;


;;
;; Definición del package
;; ----------------------

(defpackage "PT-AGENTES-SIMPLES"
  (:nicknames "PT-A-S")
  (:use "COMMON-LISP")
  (:export "VERSION" "DIR-AGENTE-P" "DIR-POSIBLE-P" "DIR-INC" "DIR-POSIBLE-PREF" "LIS-DIR-POSIBLE-P")
  (:documentation "Agentes para jugar al PlanTron simples, esto es, agentes que no tiene memoria y su comportamiento viene guiado en función del entorno que perciben en cada instante (excepto el agente-hormiga-borracha, que hace uso del generador de números aleatorios)."))

(in-package "PT-AGENTES-SIMPLES")


;;
;; Constante versión
;; -----------------

(defconstant version "0.4"
  "Versión del package PT-AGENTES-SIMPLES.")


;;
;; Funciones de tratamiento de direcciones y posiciones
;; ----------------------------------------------------

(defun libre-o-planta (tipo)
  "Devuelve T si TIPO (un valor devuelto por ENVIO:CONSULTA-TIPO) se corresponde a una posición vacía (NIL) o a una planta (ENVIO:PLANTA)"
  (or (not tipo) (eq tipo 'envio:planta)))

(defun dir-posible-p (pos dir)
  "Mira si es posible colocarse (sin obstáculo o rastro de agente con que colisionar) en la posición resultante de moverse en la dirección DIR a la posición POS. Devuelve DIR en tal caso y NIL en caso contrario. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. DIR ha de ser un número entero (dirección) entre 0 y 3 (norte, este, sur y oeste respectivamente)."
  (if (libre-o-planta (envio:consulta-tipo (envio:consulta-pos-tab pos dir)))
      dir))                                   ; Se puede, entonces devolver DIR

(defun dir-agente-p (pos dir)
  "Mira si hay un agente en la posición resultante de moverse en la dirección DIR a la posición POS. Devuelve DIR en tal caso y NIL en caso contrario. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. DIR ha de ser un número entero (dirección) entre 0 y 3 (norte, este, sur y oeste respectivamente)."
  (if (eq (envio:consulta-tipo                ; Consulta tipo de contenido
	   (envio:consulta-pos-tab pos dir))  ; Consulta contenido tablero POS+DIR
	  'envio:jugador)                     ; Tipo agente jugador
      dir))                                   ; Si hay agente devuelve DIR

(defun number-o-0 (elm)
  "Si ELM es un número lo devuelve, sino devuelve 0."
  (if (numberp elm)
      elm
    0))

(defun dir-inc (dir dir-ini)
  "Calcula la dirección DIR relativa a DIR-INI, esto es, gira DIR-INI pasos en sentido agujas del reloj sobre DIR. DIR y DIR-INI han de ser números enteros (direcciones, número entero entre 0 y 3: norte, este, sur y oeste respectivamente), si no se consideran cero (norte)."
  (rem (+ (number-o-0 dir)
	  (number-o-0 dir-ini))
       4))

(defun dir-posible-pref (pos lis-dir-pref &optional dir-ini)
  "Busca la primera dirección posible (ver función DIR-POSIBLE-P) alrededor de POS siguiendo el orden de LIS-DIR-PREF, si DIR-INI está instanciado las direcciones de LIS-DIR-PREF son relativas a DIR-INI. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. LIS-DIR-PREF ha de ser una lista con números enteros (direcciones). DIR-INI ha de ser NIL, direccionamiento absoluto, o un número entero (dirección), direccionamiento relativo al valor de DIR-INI."
  (if (consp lis-dir-pref)
      (or (dir-posible-p pos (dir-inc (first lis-dir-pref) dir-ini))
	  (dir-posible-pref pos (rest lis-dir-pref) dir-ini))))


;;
;; Agente mitad
;; ------------

(defun agente-mitad (id)
  "Agente jugador PlanTron que sólo busca la mitad"
  (dir-posible-pref (envio:consulta-pos-jug id) '(0 2 3 1)))


;;
;; Agente mimitad
;; --------------

(defun suma-a-todos (num lis-num)
  "Genera lista a partir de sumar NUM a todos los elementos de LIS-NUM"
  (cond
   ((endp lis-num) nil)
   (t (cons (+ num (first lis-num))
	    (suma-a-todos num (rest lis-num))))))

(defun agente-mimitad (id)
  "Agente jugador PlanTron que se va cada uno por su mitad"
  (dir-posible-pref (envio:consulta-pos-jug id)    ; Permite iniciar en distintas
		    (suma-a-todos (* (- id 1) 2) '(0 2 3 1)))) ; direcciones según ID


;;
;; Agente espiral derecha
;; ----------------------
  
(defun agente-espiral (id)
  "Agente jugador PlanTron que da vueltas en espiral desenrrollándose!"
  (dir-posible-pref (envio:consulta-pos-jug id)  ; Der, Frente, Izq
		    '(1 0 3)                     ; Lista de direcciones relativa a
		    (first (envio:consulta-est-jug id)))) ; la dirección última


;;
;; Agente muralla
;; --------------

(defun agente-muralla (id)
  "Agente jugador PlanTron que va temerariamente enrrollándose!"
  (dir-posible-pref (envio:consulta-pos-jug id)
		    '(0 1 3)                      ; Frente, Der, Izq
		    (first (envio:consulta-est-jug id)))) ; la dirección última


;;
;; Agente imitador
;; ---------------

(defun agente-imitador (id)
  "Agente jugador PlanTron que trata de hacer el mismo movimiento que su primer adversario, si tiene"
  (dir-posible-pref (envio:consulta-pos-jug id) 
		  '(0 1 3 2)   ; Igual o lo mas parecido al adversario
		  ; Extraemos dirección del primer adversario
		  (first (envio:consulta-est-jug (if (eq id 1) 2 1)))))


;;
;; Agente cobarde
;; --------------

(defun huir-p (id pos lis-dir-pref)
  "Revisa posiciones de LIS-DIR-PREF y si está pegado a algún agente trata de escapar. Si no tiene agente pegado trata de moverse com la útima vez."
  (cond
   ((endp lis-dir-pref) nil)  ; Nil es la dirección de última vez
   ((and (dir-agente-p pos (first lis-dir-pref))
	 (dir-posible-pref pos '(2 1 3) (first lis-dir-pref)))) ; Dir. contraria a adversario
   (t (huir-p id pos (rest lis-dir-pref)))))

(defun agente-cobarde (id)
  "Agente jugador PlanTron que huye de cualquiera que esté a su lado, incluso su propia sombra"
  (huir-p id (envio:consulta-pos-jug id) '(0 1 3 2)))


;;
;; Agente hormiga borracha
;; -----------------------

(defvar random-state-hormiga-borracha (make-random-state *random-state*)
  "Estado random para el AGENTE-HORMIGA-BORRACHA.")

(defun lis-dir-posible-p (pos lis-dir-pref)
  "Genera una lista con las direcciones a las que es posible ir a partir de las direcciones de LIS-DIR-PREF"
  (cond
   ((endp lis-dir-pref) nil)
   ((dir-posible-p pos (first lis-dir-pref))
    (cons (first lis-dir-pref) (lis-dir-posible-p pos (rest lis-dir-pref))))
   (t (lis-dir-posible-p pos (rest lis-dir-pref)))))

(defun elemento-aleatorio (lista)
  "Elige un elemento aleatorio de la lista LISTA"
  (if (consp lista)  ; Hay donde moverse
      (nth (random (length lista) random-state-hormiga-borracha) lista)))

(defun agente-hormiga-borracha (id)
  "Agente jugador PlanTron que da tumbos y se agarra a las farolas. Elige unmovimientro aleatoriamente entre los posibles"
  (elemento-aleatorio
   (lis-dir-posible-p (envio:consulta-pos-jug id) '(0 1 2 3))))


;;
;; Definiciones de agentes presentes en este package
;; -------------------------------------------------

(pt:defagente mitad
  :turno #'agente-mitad
  :documentation "Agente jugador PlanTron que sólo busca la mitad")

(pt:defagente mimitad
  :turno #'agente-mimitad
  :documentation "Agente jugador PlanTron que se va cada uno por su mitad")

(pt:defagente espiral
  :turno #'agente-espiral
  :documentation "Agente jugador PlanTron que da vueltas en espiral desenrrollándose!")

(pt:defagente muralla
  :turno #'agente-muralla
  :documentation "Agente jugador PlanTron que va temerariamente por los bordes")

(pt:defagente imitador
  :turno #'agente-imitador
  :documentation "Agente jugador PlanTron que trata de hacer el mismo movimiento que su primer adversario, si tiene")

(pt:defagente cobarde
  :turno #'agente-cobarde
  :documentation "Agente jugador PlanTron que huye de cualquiera que esté a su lado, incluso su propia sombra")

(pt:defagente hormiga-borracha
  :turno #'agente-hormiga-borracha
  :documentation "Agente jugador PlanTron que da tumbos y se agarra a las farolas. Elige un movimientro aleatoriamente entre los posibles")

==> plantron-1.5.5.tar.gz: plantron/arena/pt-agentes-simples.lisp <==
;; Programa: Package con agentes jugadores simples de PlanTron
;;
;; Asignatura: Introducción a la Inteligencia Artificial
;;             ETSII - ULL
;; Práctica: (lisp)
;; Profesor: Patricio García (patricio AT etsii.ull.es)
;; Fecha:    28/04/2008
;; Comentarios:
;;
;;           Incluye agentes para jugar al PlanTron simples, esto
;;           es, agentes que no tiene memoria y su comportamiento
;;           viene guiado en función del entorno que perciben en
;;           cada instante (excepto el agente-hormiga-borracha,
;;           que hace uso del generador de números aleatorios).
;;
;;           Los agentes definidos utilizan el entorno de juego
;;           PlanTron, definido principalmente por los packages
;;           PT-BASE y ENVIO.
;;
;;           Los agentes incluidos son, ordenados según la
;;           complejidad de su código, los siguientes:
;;
;;             AGENTE-MITAD
;;             AGENTE-MIMITAD
;;             AGENTE-ESPIRAL
;;             AGENTE-ROMBO
;;             AGENTE-IMITADOR
;;             AGENTE-COBARDE
;;             AGENTE-HORMIGA-BORRACHA
;;
;;           Ver la documentación existente para cada uno de ellos.
;;
;;           Para su utilización es aconsejable usar el cargador
;;           del entorno de juego PlanTron: "pt-loader.lisp".
;;
;;
;; Dependencias:
;;
;;           Hace uso del paquete que contiene la interface (ENVIO)
;;           con el control del juego (PT-BASE).
;;
;;
;; Modificaciones:
;;
;;           - Ver 0.1 (28/04/2008): Primera versión
;;
;;           - Ver 0.2 (14/04/2009): Versión curso 2008/09
;;
;;           - Ver 0.2.1 (15/04/2009): Añadidos más comentarios
;;
;;           - Ver 0.3 (24/03/2010): Versión curso 2009/10
;;
;;           - Ver 0.4 (24/03/2011): Versión curso 2010/11
;;
;;           - Ver 0.5 (28/03/2012): Versión curso 2011/12
;;
;;           - Ver 0.5.1 (02/04/2012): Incluido agente espiral
;;
;;           - Ver 0.6 (26/09/2012): Versión curso 2012/13 IAIC
;;
;; Por hacer:
;;
;;
;;


;;
;; Definición del package
;; ----------------------

(defpackage "PT-AGENTES-SIMPLES"
  (:nicknames "PT-A-S")
  (:use "COMMON-LISP")
  (:export "VERSION" "DIR-AGENTE-P" "DIR-POSIBLE-P" "DIR-INC" "DIR-POSIBLE-PREF" "LIS-DIR-POSIBLE-P")
  (:documentation "Agentes para jugar al PlanTron simples, esto es, agentes que no tiene memoria y su comportamiento viene guiado en función del entorno que perciben en cada instante (excepto el agente-hormiga-borracha, que hace uso del generador de números aleatorios)."))

(in-package "PT-AGENTES-SIMPLES")


;;
;; Constante versión
;; -----------------

(defconstant version "0.6"
  "Versión del package PT-AGENTES-SIMPLES.")


;;
;; Funciones de tratamiento de direcciones y posiciones
;; ----------------------------------------------------

(defun libre-o-planta (tipo)
  "Devuelve T si TIPO (un valor devuelto por ENVIO:CONSULTA-TIPO) se corresponde a una posición vacía (NIL) o a una planta (ENVIO:PLANTA)"
  (or (not tipo) (eq tipo 'envio:planta)))

(defun dir-posible-p (pos dir)
  "Mira si es posible colocarse (sin obstáculo o rastro de agente con que colisionar) en la posición resultante de moverse en la dirección DIR a la posición POS. Devuelve DIR en tal caso y NIL en caso contrario. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. DIR ha de ser un número entero (dirección) entre 0 y 3 (norte, este, sur y oeste respectivamente)."
  (if (libre-o-planta (envio:consulta-tipo (envio:consulta-pos-tab pos dir)))
      dir))                                   ; Se puede, entonces devolver DIR

(defun dir-agente-p (pos dir)
  "Mira si hay un agente en la posición resultante de moverse en la dirección DIR a la posición POS. Devuelve DIR en tal caso y NIL en caso contrario. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. DIR ha de ser un número entero (dirección) entre 0 y 3 (norte, este, sur y oeste respectivamente)."
  (if (eq (envio:consulta-tipo                ; Consulta tipo de contenido
	   (envio:consulta-pos-tab pos dir))  ; Consulta contenido tablero POS+DIR
	  'envio:jugador)                     ; Tipo agente jugador
      dir))                                   ; Si hay agente devuelve DIR

(defun number-o-0 (elm)
  "Si ELM es un número lo devuelve, sino devuelve 0."
  (if (numberp elm)
      elm
    0))

(defun dir-inc (dir dir-ini)
  "Calcula la dirección DIR relativa a DIR-INI, esto es, gira DIR-INI pasos en sentido agujas del reloj sobre DIR. DIR y DIR-INI han de ser números enteros (direcciones, número entero entre 0 y 3: norte, este, sur y oeste respectivamente), si no se consideran cero (norte)."
  (rem (+ (number-o-0 dir)
	  (number-o-0 dir-ini))
       4))

(defun dir-posible-pref (pos lis-dir-pref &optional dir-ini)
  "Busca la primera dirección posible (ver función DIR-POSIBLE-P) alrededor de POS siguiendo el orden de LIS-DIR-PREF, si DIR-INI está instanciado las direcciones de LIS-DIR-PREF son relativas a DIR-INI. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. LIS-DIR-PREF ha de ser una lista con números enteros (direcciones). DIR-INI ha de ser NIL, direccionamiento absoluto, o un número entero (dirección), direccionamiento relativo al valor de DIR-INI."
  (if (consp lis-dir-pref)
      (or (dir-posible-p pos (dir-inc (first lis-dir-pref) dir-ini))
	  (dir-posible-pref pos (rest lis-dir-pref) dir-ini))))

;;
;; Agente mitad
;; ------------

(defun agente-mitad (id)
  "Agente jugador PlanTron que sólo busca la mitad"
  (dir-posible-pref (envio:consulta-pos-jug id) '(0 2 1 3)))


;;
;; Agente mimitad
;; --------------

(defun suma-a-todos (num lis-num)
  "Genera lista a partir de sumar NUM a todos los elementos de LIS-NUM"
  (cond
   ((endp lis-num) nil)
   (t (cons (+ num (first lis-num))
	    (suma-a-todos num (rest lis-num))))))

(defun agente-mimitad (id)
  "Agente jugador PlanTron que se va cada uno por su mitad"
  (dir-posible-pref (envio:consulta-pos-jug id)    ; Permite iniciar en distintas
		    (suma-a-todos (* (- id 1) 2) '(0 2 3 1)))) ; direcciones según ID


;;
;; Agente espiral derecha
;; ----------------------
  
(defun agente-espiral (id)
  "Agente jugador PlanTron que da vueltas en espiral desenrrollándose!"
  (dir-posible-pref (envio:consulta-pos-jug id)  ; Der, Frente, Izq
		    '(1 0 3)                     ; Lista de direcciones relativa a
		    (first (envio:consulta-est-jug id)))) ; la dirección última


;;
;; Agente muralla
;; --------------

(defun agente-rombo (id)
  "Agente jugador PlanTron que va temerariamente enrrollándose!"
  (dir-posible-pref (envio:consulta-pos-jug id)
		    '(1 3 0)                      ; Der, Izq, Frente
		    (first (envio:consulta-est-jug id)))) ; la dirección última


;;
;; Agente imitador
;; ---------------

(defun agente-imitador (id)
  "Agente jugador PlanTron que trata de hacer el mismo movimiento que su primer adversario, si tiene"
  (dir-posible-pref (envio:consulta-pos-jug id) 
		  '(0 1 3 2)   ; Igual o lo más parecido al adversario
		  ; Extraemos dirección del primer adversario
		  (first (envio:consulta-est-jug (if (eq id 1) 2 1)))))


;;
;; Agente cobarde
;; --------------

(defun huir-p (id pos lis-dir-pref)
  "Revisa posiciones de LIS-DIR-PREF y si está pegado a algún agente trata de escapar. Si no tiene agente pegado se mueve como agente espiral."
  (cond
   ((endp lis-dir-pref) (agente-espiral id))  ; 
   ((and (dir-agente-p pos (first lis-dir-pref))
	 (dir-posible-pref pos '(2 3 1) (first lis-dir-pref)))) ; Dir. contraria a adversario
   (t (huir-p id pos (rest lis-dir-pref)))))

(defun agente-cobarde (id)
  "Agente jugador PlanTron que huye de cualquiera que esté a su lado, incluso su propia sombra"
  (huir-p id (envio:consulta-pos-jug id) '(0 1 3 2)))


;;
;; Agente hormiga borracha
;; -----------------------

(defvar random-state-hormiga-borracha (make-random-state *random-state*)
  "Estado random para el AGENTE-HORMIGA-BORRACHA")

(defun lis-dir-posible-p (pos lis-dir-pref)
  "Genera una lista con las direcciones a las que es posible ir a partir de las direcciones de LIS-DIR-PREF"
  (cond
   ((endp lis-dir-pref) nil)
   ((dir-posible-p pos (first lis-dir-pref))
    (cons (first lis-dir-pref) (lis-dir-posible-p pos (rest lis-dir-pref))))
   (t (lis-dir-posible-p pos (rest lis-dir-pref)))))

(defun elemento-aleatorio (lista)
  "Elige un elemento aleatorio de la lista LISTA"
  (if (consp lista)  ; Hay donde moverse
      (nth (random (length lista) random-state-hormiga-borracha) lista)))

(defun agente-hormiga-borracha (id)
  "Agente jugador PlanTron que da tumbos y se agarra a las farolas. Elige un movimientro aleatoriamente entre los posibles"
  (elemento-aleatorio
   (lis-dir-posible-p (envio:consulta-pos-jug id) '(0 1 2 3))))


;;
;; Definiciones de agentes presentes en este package
;; -------------------------------------------------

(pt:defagente mitad
  :turno #'agente-mitad
  :documentation "Agente jugador PlanTron que sólo busca la mitad")

(pt:defagente mimitad
  :turno #'agente-mimitad
  :documentation "Agente jugador PlanTron que se va cada uno por su mitad")

(pt:defagente espiral
  :turno #'agente-espiral
  :documentation "Agente jugador PlanTron que da vueltas en espiral desenrrollándose!")

(pt:defagente rombo
  :turno #'agente-rombo
  :documentation "Agente jugador PlanTron que va temerariamente por enrollándose")

(pt:defagente imitador
  :turno #'agente-imitador
  :documentation "Agente jugador PlanTron que trata de hacer el mismo movimiento que su primer adversario, si tiene")

(pt:defagente cobarde
  :turno #'agente-cobarde
  :documentation "Agente jugador PlanTron que huye de cualquiera que esté a su lado, incluso su propia sombra")

(pt:defagente hormiga-borracha
  :turno #'agente-hormiga-borracha
  :documentation "Agente jugador PlanTron que da tumbos y se agarra a las farolas. Elige un movimientro aleatoriamente entre los posibles")
